@@ -241,9 +241,16 @@ func (r *ReconcileVirtualMachineImport) Reconcile(request reconcile.Request) (re
 	reqLogger := log.WithValues("Request.Namespace", request.Namespace, "Request.Name", request.Name)
 	reqLogger.Info("Reconciling VirtualMachineImport")
 
+	config, err := r.ctrlConfigProvider.GetConfig()
+	if err != nil {
+		log.Error(err, "Cannot get controller config.")
+	} else {
+		r.ctrlConfig = config
+	}
+
 	// Fetch the VirtualMachineImport instance
 	instance := &v2vv1.VirtualMachineImport{}
-	err := r.client.Get(context.TODO(), request.NamespacedName, instance)
+	err = r.client.Get(context.TODO(), request.NamespacedName, instance)
 	if err != nil {
 		if k8serrors.IsNotFound(err) {
 			// Request object not found, could have been deleted after reconcile request.
@@ -312,11 +319,20 @@ func (r *ReconcileVirtualMachineImport) Reconcile(request reconcile.Request) (re
 	}
 	defer provider.Close()
 
-	if instance.DeletionTimestamp != nil && utils.HasFinalizer(instance, utils.RestoreVMStateFinalizer) {
-		err := r.finalize(instance, provider)
-		if err != nil {
-			// requeue if locked
-			return reconcile.Result{RequeueAfter: 1 * time.Second}, nil
+	if instance.DeletionTimestamp != nil {
+		if utils.HasFinalizer(instance, utils.RestoreVMStateFinalizer) {
+			err := r.finalize(instance, provider)
+			if err != nil {
+				// requeue if locked
+				return reconcile.Result{RequeueAfter: 1 * time.Second}, nil
+			}
+		}
+		if utils.HasFinalizer(instance, utils.CleanupSnapshotsFinalizer) && instance.Status.WarmImport.RootSnapshot != nil {
+			_ = provider.RemoveVMSnapshot(*instance.Status.WarmImport.RootSnapshot, true)
+			err = utils.RemoveFinalizer(instance, utils.CleanupSnapshotsFinalizer, r.client)
+			if err != nil {
+				reqLogger.Error(err, "Finalizing - failed to remove snapshot finalizer")
+			}
 		}
 		return reconcile.Result{}, nil
 	}
@@ -344,6 +360,18 @@ func (r *ReconcileVirtualMachineImport) Reconcile(request reconcile.Request) (re
 
 	// don't stop the VM during a warm import unless it's time to finalize
 	if !shouldWarmImport(provider, instance) || shouldFinalizeWarmImport(instance) {
+		if _, ok := instance.Annotations[sourceVMInitialState]; !ok {
+			vmStatus, err := provider.GetVMStatus()
+			if err != nil {
+				return reconcile.Result{}, err
+			}
+
+			log.Info("Storing source VM status", "status", vmStatus)
+			err = r.storeSourceVMStatus(instance, string(vmStatus))
+			if err != nil {
+				return reconcile.Result{}, err
+			}
+		}
 		// Stop the VM
 		if err = provider.StopVM(instance, r.client); err != nil {
 			return reconcile.Result{}, err
@@ -397,6 +425,27 @@ func (r *ReconcileVirtualMachineImport) Reconcile(request reconcile.Request) (re
 		}
 
 		if !done {
+			// if the datavolumes are waiting for first consumer, then
+			// attempt to start the VM so that Kubevirt can schedule
+			// it and allow the datavolumes to bind.
+			waiting, err := r.dvsWaitingForFirstConsumer(instance, mapper, vmName)
+			if err != nil {
+				return reconcile.Result{RequeueAfter: FastReQ}, err
+			}
+			if waiting {
+				log.Info("Waiting for data volumes to be bound.")
+				err = r.setRunning(vmName, true)
+				if err != nil {
+					return reconcile.Result{RequeueAfter: FastReQ}, err
+				}
+			} else {
+				// restore the original running state if the datavolumes are no longer waiting.
+				err = r.setRunning(vmName, mapper.RunningState())
+				if err != nil {
+					return reconcile.Result{RequeueAfter: FastReQ}, err
+				}
+			}
+
 			reqLogger.Info("Waiting for disks to be imported")
 			return reconcile.Result{RequeueAfter: SlowReQ}, nil
 		}
@@ -573,11 +622,11 @@ func (r *ReconcileVirtualMachineImport) importDisks(provider provider.Provider,
 			}
 			if valid {
 				log.Info("Creating data volume", "DataVolume.Name", dv.Name, "VM.Name", vmName)
-				if _, err = r.createDataVolume(provider, mapper, instance, &dv, vmName); err != nil {
-					if err = r.endDiskImportFailed(provider, instance, foundDv, err.Error()); err != nil {
+				if _, createErr := r.createDataVolume(provider, mapper, instance, &dv, vmName); createErr != nil {
+					if err = r.endDiskImportFailed(provider, instance, foundDv, createErr.Error()); err != nil {
 						return false, err
 					}
-					return false, err
+					return false, createErr
 				}
 			} else {
 				// If disk status is wrong, end the import as failed:
@@ -664,6 +713,28 @@ func (r *ReconcileVirtualMachineImport) importDisks(provider provider.Provider,
 	return done, nil
 }
 
+func (r *ReconcileVirtualMachineImport) dvsWaitingForFirstConsumer(instance *v2vv1.VirtualMachineImport, mapper provider.Mapper, vmName types.NamespacedName) (bool, error) {
+	dvs, err := mapper.MapDataVolumes(&vmName.Name, r.filesystemOverhead)
+	if err != nil {
+		return false, err
+	}
+
+	for dvID, _ := range dvs {
+		dvName := types.NamespacedName{Namespace: instance.Namespace, Name: dvID}
+		dv, err := r.getDataVolume(dvName)
+		if err != nil {
+			return false, err
+		}
+		if dv == nil {
+			continue
+		}
+		if dv.Status.Phase == cdiv1.WaitForFirstConsumer {
+			return true, nil
+		}
+	}
+	return false, nil
+}
+
 func (r *ReconcileVirtualMachineImport) fail(provider provider.Provider, instance *v2vv1.VirtualMachineImport, reason v2vv1.SucceededConditionReason, message string) error {
 	instanceNamespacedName := types.NamespacedName{Name: instance.Name, Namespace: instance.Namespace}
 
@@ -913,7 +984,7 @@ func (r *ReconcileVirtualMachineImport) startVM(provider provider.Provider, inst
 				return false, err
 			}
 			log.Info("Starting a vm", "VM.Name", vmName)
-			if err = r.updateToRunning(vmName); err != nil {
+			if err = r.setRunning(vmName, true); err != nil {
 				// Emit event vm failed to start:
 				r.recorder.Eventf(instance, corev1.EventTypeWarning, EventVMStartFailed, "Virtual Machine %s failed to start: %s", vmIdentifier, err)
 				return false, err
@@ -972,7 +1043,11 @@ func shouldReconcile(instance *v2vv1.VirtualMachineImport) bool {
 }
 
 func shouldStartVM(instance *v2vv1.VirtualMachineImport) bool {
-	return instance.Spec.StartVM != nil && *instance.Spec.StartVM && conditions.HasSucceededConditionOfReason(instance.Status.Conditions, v2vv1.VirtualMachineReady)
+	return conditions.HasSucceededConditionOfReason(instance.Status.Conditions, v2vv1.VirtualMachineReady) &&
+		((instance.Spec.StartVM != nil && *instance.Spec.StartVM) ||
+			(instance.Spec.StartVM == nil &&
+				instance.Spec.Warm &&
+				instance.Annotations[sourceVMInitialState] == string(provider.VMStatusUp)))
 }
 
 func shouldConvertGuest(provider provider.Provider, instance *v2vv1.VirtualMachineImport) bool {
@@ -1110,7 +1185,7 @@ func (r *ReconcileVirtualMachineImport) createProvider(vmi *v2vv1.VirtualMachine
 	return nil, fmt.Errorf("Invalid source type. Only Ovirt and Vmware type is supported")
 }
 
-func (r *ReconcileVirtualMachineImport) updateToRunning(vmName types.NamespacedName) error {
+func (r *ReconcileVirtualMachineImport) setRunning(vmName types.NamespacedName, running bool) error {
 	var vm kubevirtv1.VirtualMachine
 	err := r.client.Get(context.TODO(), vmName, &vm)
 	if err != nil {
@@ -1118,7 +1193,6 @@ func (r *ReconcileVirtualMachineImport) updateToRunning(vmName types.NamespacedN
 	}
 
 	copy := vm.DeepCopy()
-	running := true
 	copy.Spec.Running = &running
 
 	patch := client.MergeFrom(&vm)
@@ -1462,7 +1536,7 @@ func validateName(instance *v2vv1.VirtualMachineImport, sourceName string) error
 		name = sourceName
 	}
 
-	errs := k8svalidation.IsQualifiedName(name)
+	errs := k8svalidation.IsDNS1123Label(name)
 	if len(errs) != 0 {
 		var errString string
 		for _, e := range errs {
@@ -1533,17 +1607,6 @@ func (r *ReconcileVirtualMachineImport) validate(instance *v2vv1.VirtualMachineI
 			r.recorder.Eventf(instance, corev1.EventTypeNormal, EventImportBlocked, "Virtual Machine %s/%s import blocked: %s", instance.Namespace, vmName, message)
 			return false, nil
 		}
-
-		vmStatus, err := provider.GetVMStatus()
-		if err != nil {
-			return true, err
-		}
-
-		logger.Info("Storing source VM status", "status", vmStatus)
-		err = r.storeSourceVMStatus(instance, string(vmStatus))
-		if err != nil {
-			return true, err
-		}
 	} else {
 		logger.Info("VirtualMachineImport has already been validated positively. Skipping re-validation")
 	}
